require "Window"
require "ChatSystemLib"
require "Item"
 
-----------------------------------------------------------------------------------------------
-- AuctionHouseBLFilter Module Definition
----------------------------------------------------------------------------------------------- 
local AuctionHouseBLFilter = Apollo.GetPackage("Gemini:Addon-1.1").tPackage:NewAddon("AuctionHouseBLFilter", false, {"MarketplaceAuction"}, "Gemini:Hook-1.0")

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local ktVersion = 1

local ktDefaultSettings = {
	nVersion = ktVersion,
	tBlackList = {}
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function AuctionHouseBLFilter:OnInitialize()
	self.tSettings = CopyTable(ktDefaultSettings)
	
	self.xmlDoc = XmlDoc.CreateFromFile("AuctionHouseBLFilter.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function AuctionHouseBLFilter:OnDocLoaded()
	if self.xmlDoc == nil or not self.xmlDoc:IsLoaded() then
		Apollo.AddAddonErrorText(self, "XmlDoc not loaded.")
	end

	self.oMarketplaceAuction = Apollo.GetAddon("MarketplaceAuction")

	if not self.oMarketplaceAuction then
		Apollo.AddAddonErrorText(self, "Could not find MarketplaceAuction.")
	end

	self.bAuctionHouseHooked = self:InstallAuctionHouseHook()

	if not self.bAuctionHouseHooked then
		Apollo.AddAddonErrorText(self, "Could not hook into MarketplaceAuction.")
	end
end

function AuctionHouseBLFilter:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Realm then
		return
	end

	self.tSettings.tVersion = CopyTable(ktVersion)

	return self.tSettings
end

function AuctionHouseBLFilter:OnRestore(eLevel, tData)
	if tData ~= nil then
		self.tSettings = MergeTables(self.tSettings, tData)
	end
end

-----------------------------------------------------------------------------------------------
-- Hook Functions
-----------------------------------------------------------------------------------------------
function AuctionHouseBLFilter:InstallAuctionHouseHook()
	self:PostHook(self.oMarketplaceAuction, "Initialize"                , "Hook_Initialize"                )
	self:RawHook (self.oMarketplaceAuction, "OnItemAuctionSearchResults", "Hook_OnItemAuctionSearchResults")
	self:PostHook(self.oMarketplaceAuction, "OnRowSelectBtnCheck"       , "Hook_OnRowSelectBtnCheck"       )
	self:PostHook(self.oMarketplaceAuction, "OnRowSelectBtnUncheck"     , "Hook_OnRowSelectBtnUncheck"     )

	return true
end

function AuctionHouseBLFilter:Hook_Initialize(luaCaller)
	local oMarketplaceAuction = luaCaller

	self.BlacklistTopButtonsWindow = Apollo.LoadForm(self.xmlDoc, "BlacklistTopButtonsWindow", oMarketplaceAuction.wndMain, self)
	self.BlacklistBuyBottomButtonsWindow = Apollo.LoadForm(self.xmlDoc, "BlacklistBuyBottomButtonsWindow", oMarketplaceAuction.wndMain:FindChild("BuyBottomContainer"), self)

	local btnApply = self.BlacklistTopButtonsWindow:FindChild("ApplyBlacklistButton")

	if btnApply then
		btnApply:SetTooltip("Apply Black Listed Items To Search Results,\nUncheck To Disable Filtring.\n \nNote that blacklisted items applies per page and not to the entire search results due to server only returning paginated results.")
	end
end

function AuctionHouseBLFilter:Hook_OnItemAuctionSearchResults(luaCaller, nPage, nTotalResults, tAuctions)
	local oMarketplaceAuction = luaCaller
	
	-- This is the second stage of drawing, where we have received data for the windows from code
	if not oMarketplaceAuction.wndMain or not oMarketplaceAuction.wndMain:IsValid() then
		return
	end

	local wndParent = nil
	local bBuyTab = oMarketplaceAuction.wndMain:FindChild("HeaderBuyBtn"):IsChecked()
	if bBuyTab then
		wndParent = oMarketplaceAuction.wndMain:FindChild("BuyContainer"):FindChild("SearchResultList")
		oMarketplaceAuction.wndMain:FindChild("BuyContainer"):FindChild("WaitScreen"):Show(false)
	else
		wndParent = oMarketplaceAuction.wndMain:FindChild("SellContainer"):FindChild("SellRightSide"):FindChild("SellSimilarItemsList")
	end
	wndParent:DestroyChildren()

	--/- Filter List, applies for buy tab when buttong is checked
	if bBuyTab then
		local btnApply = self.BlacklistTopButtonsWindow:FindChild("ApplyBlacklistButton")

		if btnApply:IsChecked() then
			local tAuctionsFiltred = {}

			for idx, aucCurr in ipairs(tAuctions) do 
				local bIsOwnAuction = aucCurr:IsOwned()
				
				if bIsOwnAuction then
					table.insert(tAuctionsFiltred, aucCurr)
				else
					local itemCurr = aucCurr:GetItem()
					local itemId = itemCurr:GetItemId()
					local itemName = itemCurr:GetName()

					if TableContains(self.tSettings.tBlackList, itemId) then
						-- Print("Skipping: " .. itemId)
					else
						table.insert(tAuctionsFiltred, aucCurr)
					end
				end
			end

			tAuctions = tAuctionsFiltred
		end
	end
	--/- EoL 

	-- Main Draw
	for idx, aucCurr in ipairs(tAuctions) do
		oMarketplaceAuction:BuildListItem(aucCurr, wndParent, bBuyTab)
	end
	wndParent:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop)

	-- Pages (if buy). For sell, just ignore.
	if bBuyTab then
		oMarketplaceAuction.nCurPage = nPage
		oMarketplaceAuction.nTotalResults = nTotalResults

		-- Not found text
		local bNoResults = #wndParent:GetChildren() == 0
		if bNoResults and oMarketplaceAuction.wndMain:FindChild("FilterContainer"):FindChild("FilterClearBtn"):GetData() then
			wndParent:SetText(Apollo.GetString("Tradeskills_NoResults").."\n"..Apollo.GetString("MarketplaceAuction_TryClearingFilter"))
		elseif bNoResults then
			wndParent:SetText(Apollo.GetString("Tradeskills_NoResults"))
		else
			wndParent:SetText("")

			-- Draw page controls
			local bHasPages = nTotalResults > MarketplaceLib.kAuctionSearchPageSize
			local bIsLast = (nPage + 1) * MarketplaceLib.kAuctionSearchPageSize >= nTotalResults
			if bHasPages then
				local wndBuyPageBtnContainer = wndParent:FindChild("BuyPageBtnContainer")
				if wndBuyPageBtnContainer then
					wndBuyPageBtnContainer:Destroy()
				end
				wndBuyPageBtnContainer = Apollo.LoadForm(oMarketplaceAuction.xmlDoc, "BuyPageBtnContainer", wndParent, oMarketplaceAuction)
				wndBuyPageBtnContainer:SetText(String_GetWeaselString(Apollo.GetString("CRB_NOutOfN"), nPage + 1, math.ceil(nTotalResults / MarketplaceLib.kAuctionSearchPageSize)))
				wndBuyPageBtnContainer:FindChild("BuySearchFirstBtn"):Enable(nPage > 0)
				wndBuyPageBtnContainer:FindChild("BuySearchPrevBtn"):Enable(nPage > 0)
				wndBuyPageBtnContainer:FindChild("BuySearchNextBtn"):Enable(not bIsLast)
				wndBuyPageBtnContainer:FindChild("BuySearchLastBtn"):Enable(not bIsLast)
				wndParent:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop)
			end
		end
	end
end

function AuctionHouseBLFilter:Hook_OnRowSelectBtnCheck(luaCaller, wndHandler, wndControl)
	local oMarketplaceAuction = luaCaller

	local aucCurr = wndHandler:GetData()

	if not aucCurr then
		return
	end

	local itemCurr = aucCurr:GetItem()

	if not itemCurr then
		return
	end

	if not self.BlacklistBuyBottomButtonsWindow then
		return
	end

	local btn = self.BlacklistBuyBottomButtonsWindow:FindChild("AddtoBlacklistButton")

	if not btn then
		return
	end

	btn:Show(true)
	btn:SetData(aucCurr)

	local itemId = itemCurr:GetItemId()
	local itemName = itemCurr:GetName()

	if TableContains(self.tSettings.tBlackList, itemId) then
		btn:Enable(false)
	else
		btn:Enable(true)
	end
end

function AuctionHouseBLFilter:Hook_OnRowSelectBtnUncheck(luaCaller, wndHandler, wndControl)
	local oMarketplaceAuction = luaCaller
	
	if not self.BlacklistBuyBottomButtonsWindow then
		return
	end

	local btn = self.BlacklistBuyBottomButtonsWindow:FindChild("AddtoBlacklistButton")

	if not btn then
		return
	end

	btn:Show(false)
	btn:SetData(nil)
end

-----------------------------------------------------------------------------------------------
-- UI Functions
-----------------------------------------------------------------------------------------------
function AuctionHouseBLFilter:OnApplyBlacklistButtonCheck( wndHandler, wndControl, eMouseButton )
	self.oMarketplaceAuction:OnRefreshBtn()
end
function AuctionHouseBLFilter:OnApplyBlacklistButtonUncheck( wndHandler, wndControl, eMouseButton )
	self.oMarketplaceAuction:OnRefreshBtn()
end

function AuctionHouseBLFilter:OnOpenBlacklistButtonSignal( wndHandler, wndControl, eMouseButton ) 
	if self.BlacklistDisplayWindow then
		self.BlacklistDisplayWindow:Show(true)
	else
		self.BlacklistDisplayWindow = Apollo.LoadForm(self.xmlDoc, "BlacklistDisplayWindow", nil, self)

		self.BlacklistDisplayWindow:Show(true)
	end
	
	self.BlacklistDisplayWindow:FindChild("Version"):SetText("AHBL 0.0." .. self.tSettings.nVersion)

	self:RenderBlackListedItems()
end

function AuctionHouseBLFilter:OnAddtoBlacklistButtonSignal( wndHandler, wndControl, eMouseButton )
	local aucCurr = wndHandler:GetData()

	if not aucCurr then
		return
	end

	local itemCurr = aucCurr:GetItem()

	if not itemCurr then
		return
	end

	local itemId = itemCurr:GetItemId()
	local itemName = itemCurr:GetName()

	if TableContains(self.tSettings.tBlackList, itemId) then
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Item already exists in blacklist: [" .. itemId .. "] " .. itemName)
	else
		table.insert(self.tSettings.tBlackList, itemId)

		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Item added to blacklist: [" .. itemId .. "] " .. itemName)

		-- self.oMarketplaceAuction:OnRefreshBtn()
	end

	wndHandler:Enable(false)
end

function AuctionHouseBLFilter:OnCloseButtonSignal( wndHandler, wndControl, eMouseButton )
	if self.BlacklistDisplayWindow then
		self.BlacklistDisplayWindow:Show(false)
	end
end

function AuctionHouseBLFilter:OnClearButtonSignal( wndHandler, wndControl, eMouseButton )
	self.tSettings.tBlackList = {}

	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Blacklist cleared.")

	self:RenderBlackListedItems()
end

function AuctionHouseBLFilter:RenderBlackListedItems()
	local container = self.BlacklistDisplayWindow:FindChild("ItemsList")

	if not container then
		return
	end

	local vScrollPos = container:GetVScrollPos()

	container:DestroyChildren()
	container:SetVScrollPos(0)

	for _, itemId in pairs(self.tSettings.tBlackList) do
		local row = Apollo.LoadForm(self.xmlDoc, "BlacklistItem", container, self)
		local item = Item.GetDataFromId(itemId)

		if row and item then
			row:FindChild("ListIcon"):SetSprite(item:GetIcon())
			row:FindChild("ListName"):SetText(item:GetName())
			row:FindChild("ListID"):SetText("ID: " .. item:GetItemId())

			row:FindChild("RemoveBtn"):SetData(item)
		end
	end

	container:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop)

	if vScrollPos then
		container:SetVScrollPos(vScrollPos)
	end
end

function AuctionHouseBLFilter:OnRemoveFromBlackListButtonSignal( wndHandler, wndControl, eMouseButton )
	local itemCurr = wndHandler:GetData()

	if not itemCurr then
		return
	end

	local itemId = itemCurr:GetItemId()
	local itemName = itemCurr:GetName()

	for i=#self.tSettings.tBlackList,1,-1 do
		if self.tSettings.tBlackList[i] == itemId then
			table.remove(self.tSettings.tBlackList, i)
		end
	end

	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Item removed from blacklist: [" .. itemId .. "] " .. itemName)

	self:RenderBlackListedItems()
end

-----------------------------------------------------------------------------------------------
-- Helper Functions
-----------------------------------------------------------------------------------------------
function TableContains(t, e)
	for _, v in pairs(t) do
		if v == e then
			return true
		end
	end
	
	return false
end

function CopyTable(orig)
	local orig_type = type(orig)
	local copy
	if orig_type == "table" then
		copy = {}
		for orig_key, orig_value in next, orig, nil do
			copy[CopyTable(orig_key)] = CopyTable(orig_value)
		end
		setmetatable(copy, CopyTable(getmetatable(orig)))
	else
		copy = orig
	end
	return copy
end

function MergeTables(t1, t2)
	for k, v in pairs(t2) do
		if type(v) == "table" then
			if t1[k] then
				if type(t1[k] or false) == "table" then
					MergeTables(t1[k] or {}, t2[k] or {})
				else
					t1[k] = v
				end
			else
				t1[k] = {}
				MergeTables(t1[k] or {}, t2[k] or {})
			end
		else
			t1[k] = v
		end
	end
	return t1
end
